import type { JSONRPCRequestPayload, JSONRPCErrorCallback } from 'ethereum-protocol';
export interface DashboardProviderOptions {
    /** Host of the Dashboard (default: localhost) */
    dashboardHost?: string;
    /** Port of the Dashboard (default: 24012) */
    dashboardPort?: number;
    /** Number of seconds before a dashboard-provider request times out (default: 120) */
    timeoutSeconds?: number;
    /** Boolean indicating whether the connection to the dashboard is kept alive between requests (default: false) */
    keepAlive?: boolean;
    /** Boolean indicating whether debug output should be logged (default: false) */
    verbose?: boolean;
    /** Boolean indicating whether the dashboard should automatically get opened in the default browser (default: true) */
    autoOpen?: boolean;
}
export declare class DashboardProvider {
    dashboardHost: string;
    dashboardPort: number;
    keepAlive: boolean;
    autoOpen: boolean;
    private socket;
    private timeoutSeconds;
    private concurrentRequests;
    private connecting;
    private verbose;
    constructor(options?: DashboardProviderOptions);
    send(payload: JSONRPCRequestPayload, callback: JSONRPCErrorCallback): void;
    sendAsync(payload: JSONRPCRequestPayload, callback: JSONRPCErrorCallback): void;
    terminate(): void;
    private sendInternal;
    private shouldTerminate;
    private ready;
    setupLogging(): void;
}
